//  Beispiel 19 "Web-Sniffer"
//  
//  Unter Verwendung der IoT Brick Library

#include <all_IoT.h>

int counter = 0;

String message = "";

void handleNotFound ()
{
  String argument = "";
  String msg = "";
  msg += "URI: ";
  msg += IoT_WebServer.uri();
  msg += "\nZeit: ";
  msg += IoT_NTPdate();
  msg += " - ";
  msg += IoT_NTPtime();
  msg += "\nMethode: ";
  msg += (IoT_WebServer.method() == HTTP_GET)?"GET":"POST";
  msg += "\nArgumente: ";
  msg += IoT_WebServer.args();
  msg += "\n";
  for (uint8_t i = 0; i < IoT_WebServer.args(); i++)
  {
    msg += IoT_WebServer.argName(i);
    msg += " = ";
    msg += IoT_WebServer.arg(i);
    msg += "\n";
    if (argument != "")
    {
      argument += "&";
    }
    argument += IoT_WebServer.argName(i);
    argument += "=";
    argument += IoT_WebServer.arg(i);
  }
  message = "URL: ";
  message += IoT_WLANaddress(true);
  message += IoT_WebServer.uri();
  if (argument != "")
  {
    message += "?";
    message += argument;
  }
  message += "\n";
  message += msg;
  handleRoot();
  Serial.println(replace(message, "\n", "\r"));                     // LineFeed in Return umwandeln (Terminal)
}

void setup ()
{
  IoT_Init(true);
  IoT_TerminalWaitInit(true);
  IoT_WLANautoConnect(true);                                        // Verbindung, ggf. gespeicherte, über WLAN herstellen
  IoT_NTPinit();                                                    // Sobald der Browser direkt auf das Stammverzeichnis zugreift, 
  IoT_WebServer.onNotFound(handleNotFound);
  IoT_WebServer.on("/", handleRoot);                                // führe die Funktion handleRoot (siehe unten) aus.
  IoT_WebServer.begin();                                            // ab jetzt "hört" der Server auf HTTP-Anfragen
  Serial.println("HTTP server started");
  Serial.println("");
}

void loop ()
{
  IoT_Idle();

  IoT_WebServer.handleClient();                                     // Bediene die http Anfragen
  
  if (IoT_Keypress())                                               // Wenn Taster gedrückt wird, Konfiguration zeigen
  {
    IoT_DisplayClear(10);                                           // OLED Display löschen (10 Punkt Schrift)
    String state = IoT_WLANstatus();                                // Verbindungsstatus
    if (IoT_WLANinited)                                             // Falls WLAN Verbindung erfolgreich, Status & IP im OLED Display anzeigen
    {
      IoT_DisplayDrawText(5, 0, "WLAN: " + IoT_WLANssid());         // WLAN Netzwerk Name
      if (state == "Connected")                                     // Wenn erfolgreich verbunden, dann Signalstärke hinzufügen
      {
        state = state + " (" + str(IoT_WLANrssi()) + "dBm)";        // Signalstärke
      }
      IoT_DisplayDrawText(5, 10, state);                            // Verbindungsstatus & Signalstärke
      IoT_DisplayDrawText(5, 20, "IP: " + IoT_WLANaddress(true));   // IP Adresse
      IoT_DisplayDrawText(5, 30, "GW: " + IoT_WLANgateway(true));   // IP Gateway
      IoT_DisplayDrawText(5, 40, "NT: " + IoT_WLANsubnet(true));    // IP Subnetzmaske
    }
    else                                                            // Falls WLAN Verbindung gescheitert ist, Status im OLED Display anzeigen
    {
      IoT_DisplayDrawText(5, 10, state); 
    }
  }
  else
  {
    IoT_Idle();
    if (counter % 100 == 0) //Infos seriell nicht zu oft ausgeben (ca. alle 10 Sekunden)
    {
      if (IoT_NTPeventAvail)                                        // Zeitevent triggern
      {
        // IoT_NTPprintEvent(IoT_NTPcurrentEvent);
        IoT_NTPeventAvail = false;
      }
    }
    
    IoT_DisplayClear(16);                                           // OLED Display löschen (16 Punkt Schrift)
    IoT_DisplayAlignText(TEXT_ALIGN_CENTER);                        // Zentrierte Darstellung des Textes
    if (IoT_NTPvalid())                                             // Prüfen, ob eine gültige Uhrzeit aus dem Internet empfangen wurde
    {
      IoT_DisplayDrawText(63, 0, IoT_NTPtime());
      IoT_DisplayDrawText(63, 15, IoT_NTPdate());
      IoT_DisplayDrawText(63, 45, "WLAN connected");
      IoT_DisplayAlignText(TEXT_ALIGN_LEFT);                        // Linksbündige Darstellung des Textes
      IoT_Idle();
    }
    else                                                            // "Bitte warten..." zentriert anzeigen
    {
      IoT_DisplayClear(24);                                         // OLED Display löschen (24 Punkt Schrift)
      IoT_DisplayAlignText(TEXT_ALIGN_CENTER);                      // Zentrierte Darstellung des Textes
      IoT_DisplayDrawText(64, 5, "Bitte");
      IoT_DisplayDrawText(64, 34, "warten...");
    }
  }
  IoT_DisplayUpdate();                                              // OLED-Anzeige aktualisieren
  delay(100);                                                       // 100 ms. warten
  counter++;
}

//Mit der Funktion handleRoot() wird die Website ausgeliefert sobald eine Anfrage von einem Browser eintrifft

void handleRoot()
{
  bool AutoRefresh = false;
  String content;
  String time_web = IoT_NTPtime();
  String date_web = IoT_NTPdate();
  content = "<!DOCTYPE html>";
  content += "<html>";
  content += "<head>";
  content += "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">";
  if (AutoRefresh)
  {
    content += "<meta http-equiv=\"refresh\" content=\"5; URL=http://" + IoT_WLANaddress(false) + "\">"; // Auto-Refresh
  }
  content += "<title>IoT Brick-Website</title>";
  content += "</head>";
  content += "<body>";
  content += "<h1> Web-Sniffer! </h1>";
  content += "<p>Diese Seite zeigt alle HTTP-Anfragen an, die sich nicht auf die Startseite beziehen.</p>";
  content += "<h2>Datum: "+date_web+"</h2>";
  content += "<h2>Uhrzeit: "+time_web+"</h2>";
  if (AutoRefresh)
  {
    content += "<p>Die Seite wird alle 5 Sekunden aktualisiert.</p>"; // Auto-Refresh
  }
  else
  {
    content += "<p>Durch Neuladen der Website können die Werte jederzeit aktualisiert werden.</p>";
  }
  content += "<p>&nbsp;</p>";
  content += "<p>Letzte empfangene Anfrage:</p>";
  content += "<p>";
  content += replace(message, "\n", "</p><p>");
  content += "</p>";
  content += "</body>";
  content += "</html>";
  IoT_WebServer.send(200, "text/html", content);   // HTTP-Statuscode 200 = OK (Anfrage wurde erfolgreich bearbeitet)
}

